/**
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.gui;

import holoedit.Main;
import holoedit.data.HoloRecentFile;
import holoedit.functions.Algorithm;
import holoedit.opengl.ScoreIndex;
import holoedit.util.Ut;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.IOException;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.plaf.metal.MetalLookAndFeel;

public class HoloMenuBar extends JMenuBar
{
	private JMenu fileMenu;
	private JMenu editMenu;
	private JMenu trackMenu;
	private JMenu scoreMenu;
	private JMenu optionMenu;
	private JMenu functionsMenu;
	private JMenu transportMenu;
	private JMenu hpMenu;
	private JMenu viewMenu;
	private JMenu helpMenu;
	// FILE
	private HoloMenuItem newMenuItem;
	private HoloMenuItem openMenuItem;
	private HoloMenuItem saveMenuItem;
	private HoloMenuItem saveAsMenuItem;
	private JMenu importMenu;
	private JMenu openMidiMenu;
	private HoloMenuItem openMidi7bMenuItem;
	private HoloMenuItem openMidiMenuItem;
	private HoloMenuItem openTextAllTracksMenuItem;
	private HoloMenuItem openSeqAllTracksMenuItem;
	public JMenu openRecentFilesMenu;
	private HoloMenuItem importSoundMenuItem;
	// EDIT
	private HoloMenuItem undoMenuItem;
	private HoloMenuItem redoMenuItem;
	private HoloMenuItem cutMenuItem;
	private HoloMenuItem copyMenuItem;
	private HoloMenuItem pasteMenuItem;
	private HoloMenuItem replaceMenuItem;
	private HoloMenuItem insertMenuItem;
	private HoloMenuItem eraseMenuItem;
	private HoloMenuItem selectAllMenuItem;
	// TRACK
	private HoloMenuItem init;
	private HoloMenuItem name;
	private HoloMenuItem number;
	private HoloMenuItem color;
	private HoloMenuItem add;
	private HoloMenuItem delete;
	private HoloMenuItem duplicate;
	private HoloMenuItem resetnames;
	private HoloMenuItem resetnumbers;
	private HoloMenuItem renumber;
	private HoloMenuItem reset;
	private HoloMenuItem importTrack;
	private HoloMenuItem exportTrack;
	private HoloMenuItem prevTrack;
	private HoloMenuItem nextTrack;
	private HoloMenuItem soloPrevTrack;
	private HoloMenuItem soloNextTrack;

	// SCORE
	private HoloMenuItem setCursor1;
	private HoloMenuItem setCursor2;
	private HoloMenuItem setCursor3;
	private HoloMenuItem viewAll1;
	private HoloMenuItem zoomAll1;
	private HoloMenuItem viewAll2;
	private HoloMenuItem zoomAll2;
	private HoloMenuItem viewAll3;
	private HoloMenuItem zoomAll3;
	private HoloMenuItem viewTrack;
	private HoloMenuItem viewMultiSeq;
	private HoloMenuItem viewSeq;
	private HoloMenuItem zoomTrack;
	private HoloMenuItem zoomMultiSeq;
	private HoloMenuItem zoomSeq;
	private HoloMenuItem lockTrack;
	private HoloMenuItem joinTrack;
	private HoloMenuItem joinMultiSeq;
	private HoloMenuItem trimTrack;
	private HoloMenuItem trimMultiSeq;
	private HoloMenuItem trimSeq;
	private HoloMenuItem copyTrack;
	private HoloMenuItem copyMultiSeq;
	private HoloMenuItem copySeq;
	private HoloMenuItem cuttTrack;
	private HoloMenuItem cuttMultiSeq;
	private HoloMenuItem cuttSeq;
	private HoloMenuItem paste1;
	private HoloMenuItem paste2;
	private HoloMenuItem paste3;
	private HoloMenuItem eraseTrack;
	private HoloMenuItem eraseMultiSeq;
	private HoloMenuItem eraseSeq;
	private HoloMenuItem moveTrack;
	private HoloMenuItem moveMultiSeq;
	private HoloMenuItem moveSeq;
	private HoloMenuItem cutAtTrack;
	private HoloMenuItem cutAtMultiSeq;
	private HoloMenuItem cutAtSeq;
	private HoloMenuItem importSeq;
	private HoloMenuItem importWave;
	private HoloMenuItem exportSeq;
	// OPTIONS
	private JCheckBoxMenuItem allActiveMenuItem;
	private JCheckBoxMenuItem viewOnlyEditablePointsMenuItem;
	private JCheckBoxMenuItem openLastOnLoadMenuItem;
	private JCheckBoxMenuItem viewHpsMenuItem;
	private JCheckBoxMenuItem viewSDIFoptionsMenuItem;
	private JMenu coordMenuItem;
	private ButtonGroup coord;
	private JRadioButtonMenuItem cartesMenuItem;
	private JRadioButtonMenuItem polarMenuItem;
	private JMenu lookAndFeelMenuItem;
	private ButtonGroup bg;
	public JRadioButtonMenuItem basicLAFMenuItem;
	public JRadioButtonMenuItem systemLAFMenuItem;
	public JRadioButtonMenuItem motifLAFMenuItem;
	// TRANSPORT
	private HoloMenuItem playMenuItem;
	private HoloMenuItem pauseMenuItem;
	private HoloMenuItem recMenuItem;
	private HoloMenuItem loopMenuItem;
	private HoloMenuItem spatUpdateMenuItem;
	private JCheckBoxMenuItem openOscMenuItem;
	private JCheckBoxMenuItem bonjourOscMenuItem;
	// SPEAKERS
	private HoloMenuItem addHPMenuItem;
	// VIEW
	private HoloMenuItem tkSelViewMenuItem;
	private HoloMenuItem transportViewMenuItem;
	private HoloMenuItem roomViewMenuItem;
	private HoloMenuItem timeViewMenuItem;
	private HoloMenuItem soundPoolViewMenuItem;
	private HoloMenuItem room3DViewMenuItem;
	private HoloMenuItem scoreViewMenuItem;
	private HoloMenuItem menuViewMenuItem;
	private HoloMenuItem closeViewMenuItem;
	private HoloMenuItem resetViewMenuItem;
	private HoloMenuItem resetAllViewMenuItem;
	// HELP
	private HoloMenuItem helpMenuItem;
	private HoloMenuItem docFrMenuItem;
	private HoloMenuItem docEnMenuItem;
	// WINDOWS MENUS
	private HoloMenuItem aboutMenuItem;
	private HoloMenuItem prefsMenuItem;
	private HoloMenuItem quitMenuItem;

	private Main mainRef;
	private FloatingWindow parent;
	
	public HoloMenuBar(Main s, FloatingWindow p)
	{
		mainRef = s;
		parent = p;
		HoloMenuAction aAction = new HoloMenuAction();
		// INIT_MENUS
		// Create, configure, and setup the Menubar, Menus, and Menu items.
		// ******** File Menu *******************************/
		fileMenu = new JMenu("File");
		newMenuItem = new HoloMenuItem("New tracks (Init all)", KeyEvent.VK_N, aAction);
		fileMenu.add(newMenuItem);
		fileMenu.addSeparator();
		openMenuItem = new HoloMenuItem("Open...", KeyEvent.VK_O, aAction);
		fileMenu.add(openMenuItem);
		openRecentFilesMenu = new JMenu("Open recent files");
		openRecentFilesMenu.setEnabled(false);
		fileMenu.add(openRecentFilesMenu);
		saveMenuItem = new HoloMenuItem("Save", KeyEvent.VK_S, aAction);
		fileMenu.add(saveMenuItem);
		saveAsMenuItem = new HoloMenuItem("Save As...", KeyEvent.VK_S, true, aAction);
		fileMenu.add(saveAsMenuItem);
		fileMenu.addSeparator();
		importMenu = new JMenu("Import old formats");
		// ---> import as midi
		openMidiMenu = new JMenu("Midi");
		openMidiMenuItem = new HoloMenuItem("Midi14bits", aAction);
		openMidi7bMenuItem = new HoloMenuItem("Midi7bits", aAction);
		openMidiMenu.add(openMidiMenuItem);
		openMidiMenu.add(openMidi7bMenuItem);
		importMenu.add(openMidiMenu);
		// ---> import as text
		openTextAllTracksMenuItem = new HoloMenuItem("Import Text...", aAction);
		importMenu.add(openTextAllTracksMenuItem);
		// ---> open as Seq
		openSeqAllTracksMenuItem = new HoloMenuItem("Import Seq~...", aAction);
		importMenu.add(openSeqAllTracksMenuItem);
		fileMenu.add(importMenu);
		fileMenu.addSeparator();
		importSoundMenuItem = new HoloMenuItem("Import Sound...", KeyEvent.VK_I, true, aAction);
		fileMenu.add(importSoundMenuItem);
		// WINDOWS MENUS
		prefsMenuItem = new HoloMenuItem("Preferences...",KeyEvent.VK_COMMA, aAction);
		quitMenuItem = new HoloMenuItem("Quit...",KeyEvent.VK_F4,true,true,true,true, aAction);
		if(!Ut.MAC)
		{
			fileMenu.addSeparator();
			fileMenu.add(prefsMenuItem);
			fileMenu.addSeparator();
			fileMenu.add(quitMenuItem);
		}

		// ******** Edition Menu *****************************/
		editMenu = new JMenu("Edit");
		undoMenuItem = new HoloMenuItem("Undo", KeyEvent.VK_Z, aAction);
		editMenu.add(undoMenuItem);
		redoMenuItem = new HoloMenuItem("Redo", KeyEvent.VK_Z, true, aAction);
		editMenu.add(redoMenuItem);
		editMenu.addSeparator();
		cutMenuItem = new HoloMenuItem("Cut", KeyEvent.VK_X, aAction);
		editMenu.add(cutMenuItem);
		copyMenuItem = new HoloMenuItem("Copy", KeyEvent.VK_C, aAction);
		editMenu.add(copyMenuItem);
		pasteMenuItem = new HoloMenuItem("Paste", KeyEvent.VK_V, aAction);
		editMenu.add(pasteMenuItem);
		replaceMenuItem = new HoloMenuItem("Replace", KeyEvent.VK_R, aAction);
		editMenu.add(replaceMenuItem);
		insertMenuItem = new HoloMenuItem("Insert", KeyEvent.VK_I, aAction);
		editMenu.add(insertMenuItem);
		eraseMenuItem = new HoloMenuItem("Erase", KeyEvent.VK_E, aAction);
		editMenu.add(eraseMenuItem);
		editMenu.addSeparator();
		selectAllMenuItem = new HoloMenuItem("Select All", KeyEvent.VK_A, aAction);
		editMenu.add(selectAllMenuItem);
		
		// ******** Track Menu *************/
		
		trackMenu = new JMenu("Track");
		init = new HoloMenuItem("Init", KeyEvent.VK_I, true, aAction);
		name = new HoloMenuItem("Set Name...", aAction);
		number = new HoloMenuItem("Set Number...", aAction);
		color = new HoloMenuItem("Set Color...", aAction);
		lockTrack = new HoloMenuItem("Lock this track", KeyEvent.VK_L, aAction);
		add = new HoloMenuItem("New Track", KeyEvent.VK_N, true, aAction);
		delete = new HoloMenuItem("Delete", KeyEvent.VK_D, true, aAction);
		duplicate = new HoloMenuItem("Duplicate", KeyEvent.VK_D, aAction);
		resetnames = new HoloMenuItem("Reset all names", aAction);
		resetnumbers = new HoloMenuItem("Reset all numbers", aAction);
		renumber = new HoloMenuItem("Renumber all tracks...", aAction);
		reset = new HoloMenuItem("Reset both", aAction);
		importTrack = new HoloMenuItem("Import Track...", aAction);
		exportTrack = new HoloMenuItem("Export Track...", aAction);
		prevTrack = new HoloMenuItem("Previous Track", KeyEvent.VK_UP, aAction);
		nextTrack = new HoloMenuItem("Next Track", KeyEvent.VK_DOWN, aAction);
		soloPrevTrack = new HoloMenuItem("Solo Previous Track", KeyEvent.VK_UP, false, false, false, aAction);
		soloNextTrack = new HoloMenuItem("Solo Next Track", KeyEvent.VK_DOWN, false, false, false, aAction);
		
		trackMenu.add(init);
		trackMenu.add(name);
		trackMenu.add(number);
		trackMenu.add(color);
		trackMenu.addSeparator();
		trackMenu.add(lockTrack);
		trackMenu.addSeparator();
		trackMenu.add(add);
		trackMenu.add(delete);
		trackMenu.add(duplicate);
		trackMenu.addSeparator();
		trackMenu.add(resetnames);
		trackMenu.add(resetnumbers);
		trackMenu.add(renumber);
		trackMenu.add(reset);
		trackMenu.addSeparator();
		trackMenu.addSeparator();
		trackMenu.add(importTrack);
		trackMenu.add(exportTrack);
		trackMenu.addSeparator();
		trackMenu.add(prevTrack);
		trackMenu.add(nextTrack);
		trackMenu.add(soloPrevTrack);
		trackMenu.add(soloNextTrack);

		// ******** Score Menu *************/
		scoreMenu = new JMenu("Score");
		setCursor1 = new HoloMenuItem("Set cursor at time...", aAction);
		setCursor2 = new HoloMenuItem("Set cursor at time...", aAction);
		setCursor3 = new HoloMenuItem("Set cursor at time...", aAction);
		viewAll1 = new HoloMenuItem("View All", KeyEvent.VK_F, true, aAction);
		viewAll2 = new HoloMenuItem("View All", KeyEvent.VK_F, true, aAction);
		viewAll3 = new HoloMenuItem("View All", KeyEvent.VK_F, true, aAction);
		viewTrack = new HoloMenuItem("View this track", KeyEvent.VK_F, aAction);
		viewMultiSeq = new HoloMenuItem("View this selection", KeyEvent.VK_F, aAction);
		viewSeq = new HoloMenuItem("View this trajectory", KeyEvent.VK_F, aAction);
		zoomAll1 = new HoloMenuItem("Zoom on all", KeyEvent.VK_G, true, aAction);
		zoomAll2 = new HoloMenuItem("Zoom on all", KeyEvent.VK_G, true, aAction);
		zoomAll3 = new HoloMenuItem("Zoom on all", KeyEvent.VK_G, true, aAction);
		zoomTrack = new HoloMenuItem("Zoom on this track", KeyEvent.VK_G, aAction);
		zoomMultiSeq = new HoloMenuItem("Zoom on this selection", KeyEvent.VK_G, aAction);
		zoomSeq = new HoloMenuItem("Zoom on this trajectory", KeyEvent.VK_G, aAction);
		joinTrack = new HoloMenuItem("Join all trajectories in this track", KeyEvent.VK_J, aAction);
		joinMultiSeq = new HoloMenuItem("Join selected trajectories", KeyEvent.VK_J, aAction);
		trimTrack = new HoloMenuItem("Trim this track to current time selection", KeyEvent.VK_T, aAction);
		trimMultiSeq = new HoloMenuItem("Trim this selection to current time selection", KeyEvent.VK_T, aAction);
		trimSeq = new HoloMenuItem("Trim this trajectory to current time selection", KeyEvent.VK_T, aAction);
		copyTrack = new HoloMenuItem("Copy this track", KeyEvent.VK_C, true, aAction);
		copyMultiSeq = new HoloMenuItem("Copy this selection", KeyEvent.VK_C, true, aAction);
		copySeq = new HoloMenuItem("Copy this trajectory", KeyEvent.VK_C, true, aAction);
		cuttTrack = new HoloMenuItem("Cut this track", KeyEvent.VK_X, true, aAction);
		cuttMultiSeq = new HoloMenuItem("Cut this selection", KeyEvent.VK_X, true, aAction);
		cuttSeq = new HoloMenuItem("Cut this trajectory", KeyEvent.VK_X, true, aAction);
		paste1 = new HoloMenuItem("Paste", KeyEvent.VK_V, true, aAction);
		paste2 = new HoloMenuItem("Paste", KeyEvent.VK_V, true, aAction);
		paste3 = new HoloMenuItem("Paste", KeyEvent.VK_V, true, aAction);
		eraseTrack = new HoloMenuItem("Erase this track", KeyEvent.VK_E, true, aAction);
		eraseMultiSeq = new HoloMenuItem("Erase this selection", KeyEvent.VK_E, true, aAction);
		eraseSeq = new HoloMenuItem("Erase this trajectory", KeyEvent.VK_E, true, aAction);
		moveTrack = new HoloMenuItem("Move this track to time...", KeyEvent.VK_M, aAction);
		moveMultiSeq = new HoloMenuItem("Move this selection to time...", KeyEvent.VK_M, aAction);
		moveSeq = new HoloMenuItem("Move this trajectory to time...", KeyEvent.VK_M, aAction);
		cutAtTrack = new HoloMenuItem("Cut this track at time...",KeyEvent.VK_K, aAction);
		cutAtMultiSeq = new HoloMenuItem("Cut this selection at time...",KeyEvent.VK_K, aAction);
		cutAtSeq = new HoloMenuItem("Cut this trajectory at time...",KeyEvent.VK_K, aAction);
		importSeq = new HoloMenuItem("Import Trajectory...", aAction);
		importWave = new HoloMenuItem("Import WaveForm from SoundPool...", aAction);
		exportSeq = new HoloMenuItem("Export Trajectory...", aAction);
		scoreMenu.add(setCursor1);
		scoreMenu.addSeparator();
		scoreMenu.add(viewAll1);
		scoreMenu.add(zoomAll1);
		scoreMenu.addSeparator();
		scoreMenu.add(viewTrack);
		scoreMenu.add(zoomTrack);
		scoreMenu.addSeparator();
		scoreMenu.add(joinTrack);
		scoreMenu.add(trimTrack);
		scoreMenu.add(moveTrack);
		scoreMenu.add(cutAtTrack);
		scoreMenu.addSeparator();
		scoreMenu.add(copyTrack);
		scoreMenu.add(cuttTrack);
		scoreMenu.add(paste1);
		scoreMenu.add(eraseTrack);
		scoreMenu.addSeparator();
		scoreMenu.add(importSeq);
		scoreMenu.add(importWave);
		// ******** Option Menu *************/
		optionMenu = new JMenu("Options");
		allActiveMenuItem = new JCheckBoxMenuItem("Automatic track selection", false);
		allActiveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, 8));
		optionMenu.add(allActiveMenuItem);
		viewOnlyEditablePointsMenuItem = new JCheckBoxMenuItem("View only editable points", false);
		viewOnlyEditablePointsMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, 8));
		optionMenu.add(viewOnlyEditablePointsMenuItem);
		viewHpsMenuItem = new JCheckBoxMenuItem("View Speakers", true);
		viewHpsMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, 8));
		optionMenu.add(viewHpsMenuItem);
		viewSDIFoptionsMenuItem = new JCheckBoxMenuItem("SDIF import options", false);
	//	viewSDIFoptionsMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, 8));
		optionMenu.add(viewSDIFoptionsMenuItem);
		optionMenu.addSeparator();
		openLastOnLoadMenuItem = new JCheckBoxMenuItem("Load last file on startup", false);
		optionMenu.add(openLastOnLoadMenuItem);
		optionMenu.addSeparator();
		coordMenuItem = new JMenu("Coordinates");
		coord = new ButtonGroup();
		cartesMenuItem = new JRadioButtonMenuItem("Cartesian", true);
		cartesMenuItem.addActionListener(aAction);
		polarMenuItem = new JRadioButtonMenuItem("Polar", false);
		polarMenuItem.addActionListener(aAction);
		coord.add(cartesMenuItem);
		coord.add(polarMenuItem);
		coordMenuItem.add(cartesMenuItem);
		coordMenuItem.add(polarMenuItem);
		optionMenu.add(coordMenuItem);
		lookAndFeelMenuItem = new JMenu("Change Look And Feel");
		bg = new ButtonGroup();
		basicLAFMenuItem = new JRadioButtonMenuItem("Metal L&F (Default)", true);
		basicLAFMenuItem.addActionListener(aAction);
		systemLAFMenuItem = new JRadioButtonMenuItem("System L&F", false);
		systemLAFMenuItem.addActionListener(aAction);
		motifLAFMenuItem = new JRadioButtonMenuItem("Motif L&F", false);
		motifLAFMenuItem.addActionListener(aAction);
		bg.add(basicLAFMenuItem);
		bg.add(systemLAFMenuItem);
		bg.add(motifLAFMenuItem);
		lookAndFeelMenuItem.add(basicLAFMenuItem);
		lookAndFeelMenuItem.add(systemLAFMenuItem);
		lookAndFeelMenuItem.add(motifLAFMenuItem);
		if(!Ut.MAC)
		{
			optionMenu.addSeparator();
			optionMenu.add(lookAndFeelMenuItem);
		}
		// ******** Functions & Speakers Menu *************/
		functionsMenu = new JMenu("Functions");
		hpMenu = new JMenu("Speakers");
		String currentCat = "";
		JMenuItem catTitle = null;
		boolean first = true;
		for(Algorithm a:mainRef.gestionPistes.algos)
		{
			if(!a.getCategory().equalsIgnoreCase("Speakers"))
			{
				if(!a.getCategory().equalsIgnoreCase(currentCat))
				{
					currentCat = a.getCategory();
					if(!first)
						functionsMenu.addSeparator();
					else 
						first = false;
					//	catTitle = new JMenuItem(currentCat);
					if (!currentCat.equalsIgnoreCase("Script"))
						catTitle = new JMenu(currentCat);
					//	catTitle.setEnabled(false);
					catTitle.setEnabled(true);
					if (!currentCat.equalsIgnoreCase("Script"))
							functionsMenu.add(catTitle);
						else
							functionsMenu.add(a.getMenu());
				}
				//	functionsMenu.add(a.getMenu());
				if (!currentCat.equalsIgnoreCase("Script"))
					catTitle.add(a.getMenu());
			} else {
				hpMenu.add(a.getMenu());
			}
		}
		hpMenu.addSeparator();
		addHPMenuItem = new HoloMenuItem("Add a speaker", aAction);
		hpMenu.add(addHPMenuItem);

		// ******** Transport Menu *************/
		transportMenu = new JMenu("Transport");
		openOscMenuItem = new JCheckBoxMenuItem("Enable Transport", false);
		if (Ut.MAC)
			openOscMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, 5));
		else openOscMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, 1));
		transportMenu.add(openOscMenuItem);
		openOscMenuItem.addItemListener(new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				if (e.getStateChange() == ItemEvent.SELECTED)
				{
					mainRef.connection.open();
					if (mainRef.connection.isOpen())
						mainRef.transport.activer();
					else mainRef.transport.desactiver();
					playMenuItem.setEnabled(true);
					pauseMenuItem.setEnabled(true);
					loopMenuItem.setEnabled(true);
				} else {
					mainRef.connection.close();
					mainRef.transport.desactiver();
					playMenuItem.setEnabled(false);
					pauseMenuItem.setEnabled(false);
					loopMenuItem.setEnabled(false);
				}
			}
		});
		bonjourOscMenuItem = new JCheckBoxMenuItem("Enable Bonjour/Zeroconf", true);
		bonjourOscMenuItem.addItemListener(new ItemListener()
		{
			public void itemStateChanged(ItemEvent e)
			{
				if (e.getStateChange() == ItemEvent.SELECTED)
				{
					boolean wasOpen = mainRef.connection.isOpen();
					if (wasOpen)
						mainRef.connection.close();
					mainRef.bonjour = true;
					if(wasOpen)
						mainRef.connection.open();
					boolean isNowOpen = mainRef.connection.isOpen();
					if (isNowOpen)
						mainRef.transport.activer();
					else mainRef.transport.desactiver();
				} else {
					boolean wasOpen = mainRef.connection.isOpen();
					if (wasOpen)
						mainRef.connection.close();
					mainRef.bonjour = false;
					if(wasOpen)
						mainRef.connection.open();
					boolean isNowOpen = mainRef.connection.isOpen();
					if (isNowOpen)
						mainRef.transport.activer();
					else mainRef.transport.desactiver();
				}
				update();
			}
		});		
		bonjourOscMenuItem.setEnabled(mainRef.bonjourInstalled);
		transportMenu.add(bonjourOscMenuItem);
		transportMenu.addSeparator();
		spatUpdateMenuItem = new HoloMenuItem("Holo-Spat update", aAction);
		transportMenu.add(spatUpdateMenuItem);
		transportMenu.addSeparator();
		playMenuItem = new HoloMenuItem("Play / Stop", KeyEvent.VK_SPACE, false, aAction);
		pauseMenuItem = new HoloMenuItem("Pause / Resume", KeyEvent.VK_SPACE, true, false, aAction);
		loopMenuItem = new HoloMenuItem("Loop", KeyEvent.VK_L, true, false, aAction);		
		recMenuItem = new HoloMenuItem("Record", KeyEvent.VK_R, true, false, aAction);		
		transportMenu.add(playMenuItem);
		transportMenu.add(pauseMenuItem);
		transportMenu.add(loopMenuItem);
		transportMenu.add(recMenuItem);
		transportMenu.setVisible(true);
		// ******** View Menu *************/
		viewMenu = new JMenu("View");
		tkSelViewMenuItem = new HoloMenuItem("Tracks", KeyEvent.VK_F3, false, aAction);
		roomViewMenuItem = new HoloMenuItem("Room Editor", KeyEvent.VK_F4, false, aAction);
		room3DViewMenuItem = new HoloMenuItem("3D Room", KeyEvent.VK_F4, true, true, aAction);
		scoreViewMenuItem = new HoloMenuItem("Score", KeyEvent.VK_F5, false, aAction);
		soundPoolViewMenuItem = new HoloMenuItem("Sound Pool", KeyEvent.VK_F5, true, true, aAction);
		transportViewMenuItem = new HoloMenuItem("Transport", KeyEvent.VK_F6, false, aAction);
		timeViewMenuItem = new HoloMenuItem("Time Editor", KeyEvent.VK_F7, false, aAction);
		menuViewMenuItem = new HoloMenuItem("Menu", KeyEvent.VK_F8, false, aAction);
		closeViewMenuItem = new HoloMenuItem("Close current window", KeyEvent.VK_W, aAction);
		resetViewMenuItem = new HoloMenuItem("Reset current window position & size", aAction);
		resetAllViewMenuItem = new HoloMenuItem("Reset all windows positions & sizes", aAction);
		viewMenu.add(tkSelViewMenuItem);
		viewMenu.add(roomViewMenuItem);
		viewMenu.add(room3DViewMenuItem);
		viewMenu.add(scoreViewMenuItem);
		viewMenu.add(soundPoolViewMenuItem);
		viewMenu.add(transportViewMenuItem);
		viewMenu.add(timeViewMenuItem);
		viewMenu.addSeparator();
		if(!Ut.MAC)
		{
			viewMenu.add(menuViewMenuItem);
			viewMenu.addSeparator();
		}
		viewMenu.add(closeViewMenuItem);
		viewMenu.add(resetViewMenuItem);
		viewMenu.add(resetAllViewMenuItem);
		// ******** Help Menu *************/
		helpMenu = new JMenu("Help");
		aboutMenuItem = new HoloMenuItem("About "+Ut.hv, aAction);
		helpMenuItem = new HoloMenuItem("Quick Help", KeyEvent.VK_F1, false, aAction);
		docFrMenuItem = new HoloMenuItem("Documentation (Fr)", aAction);
		docEnMenuItem = new HoloMenuItem("Documentation (En)", aAction);
		if(!Ut.MAC)
		{
			helpMenu.add(aboutMenuItem);
			helpMenu.addSeparator();
		}
		helpMenu.add(helpMenuItem);
		helpMenu.add(docFrMenuItem);
		helpMenu.add(docEnMenuItem);
		add(fileMenu);
		add(editMenu);
		add(trackMenu);
		add(scoreMenu);
		add(optionMenu);
		add(functionsMenu);
		add(transportMenu);
		add(hpMenu);
		add(viewMenu);
		add(helpMenu);

		// OPTIONS
		// Check Box Automatic Track Selection
		allActiveMenuItem.addItemListener(new java.awt.event.ItemListener()
		{
			public void itemStateChanged(java.awt.event.ItemEvent e)
			{
				if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED)
					mainRef.allTrackActive = true;
				else mainRef.allTrackActive = false;
				mainRef.gestionPistes.setDirty(true);
				mainRef.room.display();
			}
		});
		// Check Box Only Editable Points
		viewOnlyEditablePointsMenuItem.addItemListener(new java.awt.event.ItemListener()
		{
			public void itemStateChanged(java.awt.event.ItemEvent e)
			{
				mainRef.viewOnlyEditablePoints = (e.getStateChange() == java.awt.event.ItemEvent.SELECTED);
				mainRef.gestionPistes.setDirty(true);
				mainRef.room.display();
			}
		});
		// Check Box Only View Speakers
		viewHpsMenuItem.addItemListener(new java.awt.event.ItemListener()
		{
			public void itemStateChanged(java.awt.event.ItemEvent e)
			{
				mainRef.viewSpeakers = (e.getStateChange() == java.awt.event.ItemEvent.SELECTED);
				mainRef.room.display();
			}
		});
		
		// Check Box SDIF options
		viewSDIFoptionsMenuItem.addItemListener(new java.awt.event.ItemListener()
		{
			public void itemStateChanged(java.awt.event.ItemEvent e)
			{
				mainRef.sdifExpert = (e.getStateChange() == java.awt.event.ItemEvent.SELECTED);
				mainRef.room.display();
			}
		});
		
		// Check Box Open Last on load
		openLastOnLoadMenuItem.addItemListener(new java.awt.event.ItemListener()
		{
			public void itemStateChanged(java.awt.event.ItemEvent e)
			{
				if (e.getStateChange() == java.awt.event.ItemEvent.SELECTED)
					mainRef.openLastOnLoad = true;
				else
					mainRef.openLastOnLoad = false;
			}
		});
	}
	class HoloMenuAction implements ActionListener
	{
		public void actionPerformed(ActionEvent event)
		{
			// Impression du nom du menu pour tracage d'informations
			Object o = event.getSource();
			if (!o.getClass().toString().equalsIgnoreCase("class javax.swing.JRadioButtonMenuItem"))
				System.out.println("Command : " + o);
			if (o == newMenuItem)
			{
				mainRef.gestionPistes.reset();
				mainRef.gestionPistes.initTracks(false);
			}
			else if (o == openMenuItem)
				mainRef.gestionPistes.readHoloFile();
			else if (o == openMidiMenuItem)
				mainRef.gestionPistes.readFile("midi14");
			else if (o == openMidi7bMenuItem)
				mainRef.gestionPistes.readFile("midi7");
			else if (o == openTextAllTracksMenuItem)
				mainRef.gestionPistes.readFile("txt");
			else if (o == saveMenuItem)
				mainRef.gestionPistes.writeHoloFile();
			else if (o == saveAsMenuItem)
				mainRef.gestionPistes.writeHoloFileAs();
			else if (o == openSeqAllTracksMenuItem)
				mainRef.gestionPistes.readFile("seq");
			else if (o == undoMenuItem)
				mainRef.gestionPistes.Undo();
			else if (o == redoMenuItem)
				mainRef.gestionPistes.Redo();
			else if (o == cutMenuItem)
				mainRef.gestionPistes.Cut();
			else if (o == copyMenuItem)
				mainRef.gestionPistes.Copy();
			else if (o == pasteMenuItem)
				mainRef.gestionPistes.Paste();
			else if (o == replaceMenuItem)
				mainRef.gestionPistes.Replace();
			else if (o == insertMenuItem)
				mainRef.gestionPistes.Insert();
			else if (o == eraseMenuItem)
				mainRef.gestionPistes.Erase();
			else if (o == selectAllMenuItem)
				mainRef.gestionPistes.selectAll();
			else if (o == docFrMenuItem)
				openDoc("Fr");
			else if (o == docEnMenuItem)
				openDoc("En");
			else if (o == helpMenuItem)
				openHelp();
			else if (o == basicLAFMenuItem) {
				// Changement du Look and Feel
				System.out.println("Command : Metal Look & Feel");
				try {
					UIManager.setLookAndFeel(new MetalLookAndFeel());
					mainRef.updateUI();
					updateUI();
					mainRef.laf = 0;
					mainRef.room.display();
				} catch (Exception e) {}
			} else if (o == systemLAFMenuItem) {
				// Changement du Look and Feel
				System.out.println("Command : System Look & Feel");
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
					mainRef.updateUI();
					updateUI();
					mainRef.laf = 1;
					mainRef.room.display();
				} catch (Exception e) {}
			} else if (o == motifLAFMenuItem) {
				// Changement du Look and Feel
				System.out.println("Command : Motif Look & Feel");
				try {
					UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
					mainRef.updateUI();
					updateUI();
					mainRef.laf = 2;
					mainRef.room.display();
				} catch (Exception e){}
			} else if (o == playMenuItem)
				mainRef.connection.playstop();
			else if (o == loopMenuItem)
				mainRef.connection.loop(!mainRef.connection.getLoop());
			else if (o == recMenuItem)
				mainRef.connection.record(!mainRef.connection.isRecording());
			else if (o == pauseMenuItem)
				mainRef.connection.pause();
			else if (o == spatUpdateMenuItem)
				mainRef.connection.spatUpdate();
			else if (o == polarMenuItem) {
				// Changement du mode de coordonnees (polaire)
				System.out.println("Command : Polar coordinates");
				mainRef.coordinates = 1;
			} else if (o == cartesMenuItem) {
				// Changement du mode de coordonnees (cartesiennes)
				System.out.println("Command : Cartesian coordinates");
				mainRef.coordinates = 0;
			} else if (o == addHPMenuItem)
				mainRef.gestionPistes.addSpeaker();
			else if (o == tkSelViewMenuItem)
				mainRef.gestionPistes.ts.open();
			else if (o == transportViewMenuItem)
				mainRef.transport.open();
			else if (o == roomViewMenuItem) {
				setVisible(true);
				mainRef.room.open();
			} else if (o == timeViewMenuItem)
				mainRef.timeEditor.open();
			else if (o == soundPoolViewMenuItem)
				mainRef.soundPoolGui.open();
			else if (o == room3DViewMenuItem)
				mainRef.room3d.open();
			else if (o == importSoundMenuItem)
				mainRef.gestionPistes.importSound();
			else if (o == scoreViewMenuItem)
				mainRef.score.open();
			else if (o == closeViewMenuItem)
			{
				if (parent != null)
				{
					parent.close();
					mainRef.checkWindows();
				}
			} else if (o == resetAllViewMenuItem) {
				mainRef.room.resetPositionAndSize();
				mainRef.room3d.resetPositionAndSize();
				mainRef.score.resetPositionAndSize();
				mainRef.transport.resetPositionAndSize();
				mainRef.gestionPistes.ts.resetPositionAndSize();
				mainRef.timeEditor.resetPositionAndSize();
				mainRef.soundPoolGui.resetPositionAndSize();
			} else if (o == resetViewMenuItem) {
				if (parent != null)
					parent.resetPositionAndSize();
			}	
			else if (o == setCursor1 || o == setCursor2 || o == setCursor3)
				mainRef.score.setCursor();
			else if (o == viewAll1 || o == viewAll2 || o == viewAll3)
				mainRef.score.viewAll();
			else if (o == zoomAll1 || o == zoomAll2 || o == zoomAll3)
				mainRef.score.zoomAll();
			else if (o == viewTrack)
				mainRef.score.viewTrack();
			else if (o == zoomTrack)
				mainRef.score.zoomTrack();
			else if (o == viewMultiSeq)
				mainRef.score.viewMultiSeq();
			else if (o == zoomMultiSeq)
				mainRef.score.zoomMultiSeq();
			else if (o == viewSeq)
				mainRef.score.viewSeq();
			else if (o == zoomSeq)
				mainRef.score.zoomSeq();
			else if (o == joinTrack)
				mainRef.score.joinTrack();
			else if (o == joinMultiSeq)
				mainRef.score.joinMultiSeq();
			else if (o == trimTrack)
				mainRef.score.trimTrack();
			else if (o == trimMultiSeq)
				mainRef.score.trimMultiSeq();
			else if (o == trimSeq)
				mainRef.score.trimSeq();
			else if (o == eraseTrack)
				mainRef.score.eraseTrack();
			else if (o == eraseMultiSeq)
				mainRef.score.eraseMultiSeq();
			else if (o == eraseSeq)
				mainRef.score.eraseSeq();
			else if (o == copyTrack) {
				mainRef.score.copyTrack();
				mainRef.score.copyTrack();
			} else if (o == copyMultiSeq)
				mainRef.score.copyMultiSeq();
			else if (o == copySeq)
				mainRef.score.copySeq();
			else if (o == paste1 || o == paste2 || o == paste3)
				mainRef.score.paste(-1);
			else if (o == cuttTrack) {
				mainRef.score.copyTrack();
				mainRef.score.eraseTrack();
			} else if (o == cuttMultiSeq) {
				mainRef.score.copyMultiSeq();
				mainRef.score.eraseMultiSeq();
			} else if (o == cuttSeq) {
				mainRef.score.copySeq();
				mainRef.score.eraseSeq();
			} else if (o == moveTrack)
				mainRef.score.moveTrack();
			else if (o == moveMultiSeq)
				mainRef.score.moveMultiSeq();
			else if (o == moveSeq)
				mainRef.score.moveSeq();
			else if (o == cutAtTrack)
				mainRef.score.cutAtTrack();
			else if (o == cutAtMultiSeq)
				mainRef.score.cutAtMultiSeq();
			else if (o == cutAtSeq)
				mainRef.score.cutAtSeq();
			else if (o == init)
				mainRef.gestionPistes.initOneTrack(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == name)
				mainRef.gestionPistes.changeName(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == number)
				mainRef.gestionPistes.changeNumber(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == color)
				mainRef.gestionPistes.changeColor(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == lockTrack)
				mainRef.gestionPistes.changeLock(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == add)
				mainRef.gestionPistes.addTrack();
			else if (o == delete)
				mainRef.gestionPistes.deleteTrack(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == duplicate)
				mainRef.gestionPistes.duplicateTrack(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == resetnames)
				mainRef.gestionPistes.resetnames();
			else if (o == resetnumbers)
				mainRef.gestionPistes.resetnumbers();
			else if (o == renumber)
				mainRef.gestionPistes.changeNumbers();
			else if (o == reset)
				mainRef.gestionPistes.reset();
			else if (o == importTrack)
				mainRef.gestionPistes.importTrack(-1,0);
			else if (o == exportTrack)
				mainRef.gestionPistes.exportTrack(mainRef.gestionPistes.activeTrack);
			else if (o == importSeq)
				mainRef.gestionPistes.importSeq(mainRef.gestionPistes.getActiveTrackNb(),mainRef.gestionPistes.getActiveTrack().getLastDate());
			else if (o == importWave)
				mainRef.score.importWave(mainRef.gestionPistes.getActiveTrackNb());
			else if (o == exportSeq)
				mainRef.score.exportSeq();
			else if (o == prevTrack)
				mainRef.gestionPistes.prevTrack();
			else if (o == nextTrack)
				mainRef.gestionPistes.nextTrack();
			else if (o == soloPrevTrack)
				mainRef.gestionPistes.soloPrevTrack();
			else if (o == soloNextTrack)
				mainRef.gestionPistes.soloNextTrack();
			else if (o == aboutMenuItem)
				mainRef.about();
			else if (o == prefsMenuItem)
				mainRef.openPrefs();
			else if (o == quitMenuItem)
				mainRef.close();
			else if (o == menuViewMenuItem)
				mainRef.mainFrameOpen();
			mainRef.room.display();
			update();
		}
	};

	// --------------------------- Help -----------------------------------/
	public void openHelp()
	{
		Runtime r = Runtime.getRuntime();
		try
		{
			if (Ut.MAC)
				r.exec("open ./Documentations/help.html");
			else r.exec("cmd /c start .\\Documentations\\help.html");
		}
		catch (Exception exc)
		{
			exc.printStackTrace();
		}
	}

	public void openDoc(String lang)
	{
		Runtime r = Runtime.getRuntime();
		try
		{
			if (!Ut.MAC)
				r.exec("cmd /c start .\\Documentations\\Doc-Holophon-"+lang+"-V4.0.pdf");
			else r.exec("open ./Documentations/Doc-Holophon-"+lang+"-V4.0.pdf");
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
	}

	// Mise  jour du menu
	public void update()
	{
		if (mainRef != null)
		{
			boolean activ_non_empty = ((mainRef.gestionPistes.getActiveTrackNb() != -1) && !mainRef.gestionPistes.getActiveTrack().isEmpty()) || !mainRef.score.selIndex.isEmpty();
			cutMenuItem.setEnabled(activ_non_empty);
			copyMenuItem.setEnabled(activ_non_empty);
			eraseMenuItem.setEnabled(activ_non_empty);
			selectAllMenuItem.setEnabled(activ_non_empty);
			boolean clip_non_empty = (!mainRef.gestionPistes.copyTrack.isEmpty() || !mainRef.gestionPistes.copySeqs.isEmpty());
			pasteMenuItem.setEnabled(clip_non_empty);
			insertMenuItem.setEnabled(clip_non_empty);
			replaceMenuItem.setEnabled(clip_non_empty);
			undoMenuItem.setEnabled(!mainRef.gestionPistes.undoTrack.isEmpty() || mainRef.gestionPistes.undoAllTracks);
			redoMenuItem.setEnabled(!mainRef.gestionPistes.redoTrack.isEmpty() || mainRef.gestionPistes.redoAllTracks);
			boolean oscOpen = mainRef.connection.isOpen();
			bonjourOscMenuItem.setSelected(mainRef.bonjour);
			bonjourOscMenuItem.setEnabled(mainRef.bonjourInstalled);
			openOscMenuItem.setSelected(oscOpen);
			playMenuItem.setEnabled(oscOpen);
			pauseMenuItem.setEnabled(oscOpen);
			recMenuItem.setEnabled(oscOpen);
			loopMenuItem.setEnabled(oscOpen);
			loopMenuItem.setSelected(mainRef.connection.getLoop());
			spatUpdateMenuItem.setEnabled(oscOpen);
			functionsMenu.setEnabled(mainRef.gestionPistes.getActiveTrackNb() != -1);
			switch (mainRef.coordinates)
			{
			case 0:
				cartesMenuItem.setSelected(true);
				break;
			case 1:
				polarMenuItem.setSelected(true);
				break;
			default:
				break;
			}
			viewHpsMenuItem.setSelected(mainRef.viewSpeakers);
			viewSDIFoptionsMenuItem.setSelected(mainRef.sdifExpert);
			openLastOnLoadMenuItem.setSelected(mainRef.openLastOnLoad);
			allActiveMenuItem.setSelected(mainRef.allTrackActive);
			viewOnlyEditablePointsMenuItem.setSelected(mainRef.viewOnlyEditablePoints);
			openRecentFilesMenu.setEnabled(!mainRef.recentFiles.isEmpty());
			for (HoloRecentFile h : mainRef.recentFiles)
				openRecentFilesMenu.add(h);
			lockTrack.setText(mainRef.gestionPistes.getActiveTrackNb() != -1 && mainRef.gestionPistes.getActiveTrack().isLocked() ? "Unlock this track" : "Lock this track");
			scoreMenu.removeAll();
			if (mainRef.score.selIndex.isEmpty()) {
				ScoreIndex.decode(mainRef.score.selected);
				if (ScoreIndex.isSeq() || ScoreIndex.isWave()) {
					boolean seq = ScoreIndex.isSeq();
					trimSeq.setEnabled(seq);
					cutAtSeq.setEnabled(seq);
					exportSeq.setEnabled(seq);
					viewSeq.setText(seq ? "View this trajectory" : "View this waveform");
					zoomSeq.setText(seq ? "Zoom on this trajectory" : "Zoom on this waveform");
					moveSeq.setText(seq ? "Move this trajectory to time..." : "Move this waveform to time...");
					trimSeq.setText(seq ? "Trim this trajectory to current time selection" : "Cannot trim waveforms");
					cutAtSeq.setText(seq ? "Cut this selection at time..." : "Cannot \"cut at time\" waveforms");
					copySeq.setText(seq ? "Copy this trajectory" : "Copy this waveform");
					cuttSeq.setText(seq ? "Cut this trajectory" : "Cut this waveform");
					eraseSeq.setText(seq ? "Erase this trajectory" : "Erase this waveform");
					exportSeq.setText(seq ? "Export this trajectory..." : "Cannot export waveforms");
					scoreMenu.add(setCursor3);
					scoreMenu.addSeparator();
					scoreMenu.add(viewAll3);
					scoreMenu.add(zoomAll3);
					scoreMenu.addSeparator();
					scoreMenu.add(viewSeq);
					scoreMenu.add(zoomSeq);
					scoreMenu.addSeparator();
					scoreMenu.add(trimSeq);
					scoreMenu.add(moveSeq);
					scoreMenu.add(cutAtSeq);
					scoreMenu.addSeparator();
					scoreMenu.add(copySeq);
					scoreMenu.add(cuttSeq);
					scoreMenu.add(paste3);
					scoreMenu.add(eraseSeq);
					scoreMenu.addSeparator();
					scoreMenu.add(importSeq);
					scoreMenu.add(importWave);
					scoreMenu.add(exportSeq);
				} else {
					scoreMenu.add(setCursor1);
					scoreMenu.addSeparator();
					scoreMenu.add(viewAll1);
					scoreMenu.add(zoomAll1);
					scoreMenu.addSeparator();
					scoreMenu.add(viewTrack);
					scoreMenu.add(zoomTrack);
					scoreMenu.addSeparator();
					scoreMenu.add(joinTrack);
					scoreMenu.add(trimTrack);
					scoreMenu.add(moveTrack);
					scoreMenu.add(cutAtTrack);
					scoreMenu.addSeparator();
					scoreMenu.add(copyTrack);
					scoreMenu.add(cuttTrack);
					scoreMenu.add(paste1);
					scoreMenu.add(eraseTrack);
					scoreMenu.addSeparator();
					scoreMenu.add(importSeq);
					scoreMenu.add(importWave);
				}
			} else {
				scoreMenu.add(setCursor2);
				scoreMenu.addSeparator();
				scoreMenu.add(viewAll2);
				scoreMenu.add(zoomAll2);
				scoreMenu.addSeparator();
				scoreMenu.add(viewMultiSeq);
				scoreMenu.add(zoomMultiSeq);
				scoreMenu.addSeparator();
				scoreMenu.add(joinMultiSeq);
				scoreMenu.add(trimMultiSeq);
				scoreMenu.add(moveMultiSeq);
				scoreMenu.add(cutAtMultiSeq);
				scoreMenu.addSeparator();
				scoreMenu.add(copyMultiSeq);
				scoreMenu.add(cuttMultiSeq);
				scoreMenu.add(paste2);
				scoreMenu.add(eraseMultiSeq);
				scoreMenu.addSeparator();
				scoreMenu.add(importSeq);
				scoreMenu.add(importWave);
			}
		}
	}

	public void setParent(FloatingWindow f)
	{
		parent = f;
	}
}