/*
 *  -----------------------------------------------------------------------------
 *  
 *  Holo-Edit, spatial sound trajectories editor, part of Holophon
 *  Copyright (C) 2006 GMEM
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *  
 *  -----------------------------------------------------------------------------
 */
package holoedit.gui;

import holoedit.Main;
import holoedit.data.HoloTrack;
import holoedit.data.HoloWaveForm;
import holoedit.data.HoloExternalData;
import holoedit.data.HoloSDIFdata;
import holoedit.fileio.WaveFormReader;
import holoedit.fileio.SDIFreader;
import holoedit.fileio.TextFileReader;
import holoedit.util.MacFileManager;
import holoedit.util.Ut;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.lang.ref.SoftReference;
import java.util.List;
import java.util.Vector;
import java.util.Enumeration;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

// FEATURE SOUNDPOOL PREECOUTE OU PATCH MAX SIMPLE
public class SoundPoolGUI extends FloatingWindow implements DropTargetListener, DragGestureListener {

	private static Boolean supportsDnD;
	private Main mainRef;
	private DynamicTree soundTree;
	private DynamicTree dataTree;
	private DefaultMutableTreeNode selectedSndNode;
	private DefaultMutableTreeNode selectedDtNode;
	private JTextPane info;
	/** the popupMenu opened by a right-click on the soundTree */
	private PopupMenu popupSound;
	/** the popupMenu opened by a right-click on the dataTree */
	private PopupMenu popupData;
	private WaveFormRenderer waveView;
	private JSplitPane split, split2, split3;
	private Font font = new Font("Default", Font.PLAIN, 9);
	private MenuItem importSndMI = new MenuItem("Import soundfile...");
	private MenuItem removeSndMI = new MenuItem("Remove");
	private MenuItem updateSndMI = new MenuItem("Update");
	private MenuItem clearSndMI = new MenuItem("Clear");
	private MenuItem importDtMI = new MenuItem("Import datafile...");
	private MenuItem removeDtMI = new MenuItem("Remove");
	private Menu displayDtMI = new Menu("Display field...");
	private MenuItem updateDtMI = new MenuItem("Update");
	private MenuItem clearDtMI = new MenuItem("Clear");
	public boolean fine = false;
	public boolean done = false;
	public int error = WaveFormReader.NO_ERROR;
	public String errorFileName;
	public HoloWaveForm last;
	public HoloExternalData lastXtdt;
	private Vector<File> unfound;
	private File soundFolder;
	private File externalDataFolder;
	private File dataFolder;
	private boolean choosingfolder;
	private boolean acceptDrag = false;
	private HoloExternalData droppedExternalData = null;
	private Vector<HoloSDIFdata> sdifDataToDraw = new Vector<HoloSDIFdata>();
	private Vector<HoloSDIFdata> sdifDataToDrawFromSndTree = new Vector<HoloSDIFdata>();
	public Vector<String> doneAndFineData = new  Vector<String>();

	public SoundPoolGUI(Main owner) {
		super("Sound Pool", owner, owner.wsSoundPoolW, owner.wsSoundPoolH, owner.wlSoundPoolX, owner.wlSoundPoolY,
				owner.wbSoundPool);
		setResizable(true);
		mainRef = owner;
		soundTree = new DynamicTree("Imported Sounds", false);
		dataTree = new DynamicTree("Imported Datas", true);
		soundTree.addTreeSelectionListener(new WaveformTreeSelectionListener());
		dataTree.addTreeSelectionListener(new DataTreeSelectionListener());
		popupSound = new PopupMenu();
		popupSound.add(importSndMI);
		popupSound.add(removeSndMI);
		popupSound.addSeparator();
		popupSound.add(updateSndMI);
		popupSound.add(clearSndMI);
		soundTree.add(popupSound);
		popupData = new PopupMenu();
		popupData.add(importDtMI);
		popupData.add(removeDtMI);
		popupData.addSeparator();
		popupData.add(updateDtMI);
		popupData.add(displayDtMI);
		popupData.add(clearDtMI);
		dataTree.add(popupData);

		soundTree.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if ((e.isControlDown() && Ut.MAC) || e.getButton() == MouseEvent.BUTTON3) {
					if (selectedSndNode == null) {
						removeSndMI.setEnabled(false);
						popupSound.show(e.getComponent(), e.getX(), e.getY());
						return;
					}
					// remove autoris si le noeud n'est pas la racine
					removeSndMI.setEnabled(!selectedSndNode.isRoot());
					popupSound.show(e.getComponent(), e.getX(), e.getY());
					return;
				}
			}
		});

		dataTree.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if ((e.isControlDown() && Ut.MAC) || e.getButton() == MouseEvent.BUTTON3) {
					if (selectedDtNode == null) {
						displayDtMI.setEnabled(false);
						removeDtMI.setEnabled(false);
						popupData.show(e.getComponent(), e.getX(), e.getY());
						return;
					}
					displayDtMI.removeAll();
					if (!selectedDtNode.isRoot()) {
						HoloSDIFdata hsdifdt = (HoloSDIFdata) selectedDtNode.getUserObject();
						for (String field : hsdifdt.getFields())
							displayDtMI.add(new MenuItem(field));						
						for (int f=0; f<displayDtMI.getItemCount(); f++){
							final int index = f;
							displayDtMI.getItem(f).addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e) {
									HoloSDIFdata hsdifdt = (HoloSDIFdata) selectedDtNode.getUserObject();
									hsdifdt.setValuesToDrawIndex(index);
									hsdifdt.setDirty(true);
									updateWaveView();
								}
							});
						}
						displayDtMI.setEnabled(true);
						removeDtMI.setEnabled(true);
					}else {
						displayDtMI.setEnabled(true);
						removeDtMI.setEnabled(true);
					}
					popupData.show(e.getComponent(), e.getX(), e.getY());
					return;
				}
			}
		});

		removeSndMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!selectedSndNode.isRoot()) {
					int result = -1;
					try{
						// la selection est une waveform
						HoloWaveForm hwf = (HoloWaveForm) selectedSndNode.getUserObject();
						result = JOptionPane
						.showConfirmDialog(
								null,
								"This action will remove all instances of this waveform in the score\nand cannot be undone.\n" +
								"Are you sure ?",
								"Remove Waveform", JOptionPane.OK_CANCEL_OPTION);
						if (result == 0) {
							mainRef.modify();
							for (HoloTrack tk : mainRef.gestionPistes.tracks)
								tk.remove(hwf);
							for (HoloTrack tk : mainRef.gestionPistes.undoTracks)
								tk.remove(hwf);
							for (HoloTrack tk : mainRef.gestionPistes.redoTracks)
								tk.remove(hwf);
							mainRef.gestionPistes.undoTrack.remove(hwf);
							mainRef.gestionPistes.redoTrack.remove(hwf);
							mainRef.gestionPistes.soundPool.remove(hwf);
							soundTree.removeCurrentNode();
							selectedSndNode = null;
							mainRef.score.display();
						}

						hwf.setWaveBuffer(null);
						hwf = null;	
						System.gc();
					}catch(ClassCastException cce) {
						// la selection est une data						
						HoloExternalData hxtdt = (HoloExternalData) selectedSndNode.getUserObject();
						result = JOptionPane.showConfirmDialog(	null,
								"This action will detach the selected data from the waveform.\n",
								"Detach data", JOptionPane.OK_CANCEL_OPTION);
						if (result == 0) {
							mainRef.modify();
							hxtdt = (HoloExternalData) selectedSndNode.getUserObject();
							if (hxtdt.getFileType().equals("SDIF") || hxtdt.getFileType().equals("txt")) {
								HoloSDIFdata hsdifdt = (HoloSDIFdata) hxtdt;
								HoloWaveForm linkedWave = (HoloWaveForm) ((DefaultMutableTreeNode) selectedSndNode.getParent()).getUserObject();
								linkedWave.removeLinkedData(hsdifdt);
								sdifDataToDraw.clear();
								sdifDataToDrawFromSndTree.clear();		
								updateSoundTree((DefaultMutableTreeNode) selectedSndNode.getParent());

							} else {
								// TODO if necessary
							}
							mainRef.score.display();
						}
					}
				}
			}
		});
		
		removeDtMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int result = JOptionPane.showConfirmDialog(	null,
								"This action will remove all instances of those datas in both the soundPool and the score.\n" +
								"and cannot be undone.\n" +
								"Are you sure ?",
								"Remove Data", JOptionPane.OK_CANCEL_OPTION);
				if (result == 0) {
					mainRef.modify();
					HoloExternalData hxtdt = (HoloExternalData) selectedDtNode.getUserObject();
					if (hxtdt.getFileType().equals("SDIF") || hxtdt.getFileType().equals("TXT")) {
						HoloSDIFdata hsdifdt = (HoloSDIFdata) hxtdt;

						for (HoloTrack tk : mainRef.gestionPistes.tracks)
							tk.remove(hsdifdt, false);
						for (HoloTrack tk : mainRef.gestionPistes.undoTracks)
							tk.remove(hsdifdt, false);
						for (HoloTrack tk : mainRef.gestionPistes.redoTracks)
							tk.remove(hsdifdt, false);
						mainRef.gestionPistes.undoTrack.remove(hsdifdt, false);
						mainRef.gestionPistes.redoTrack.remove(hsdifdt, false);
										
						sdifDataToDraw.clear();
						sdifDataToDrawFromSndTree.clear();						
						// remove de la soundPool
						Vector<HoloWaveForm> sounds = mainRef.gestionPistes.soundPool.getSounds();
						for (HoloWaveForm h : sounds)
							h.removeLinkedData(hsdifdt);
						// remove de la dataPool
						mainRef.gestionPistes.externalDataPool.remove(hsdifdt);
						// TODO virer
						  // On libre l'objet : 
						hsdifdt.sdifTreeMap = null;
						hsdifdt = null;	
						System.gc();					 
					} else {
						// TODO if necessary
					}
					// si la data a remover est selectionne dans le sound tree
					// TODO pareil avec les peres du dataTree si ca entraine
					// leur remove
					if (selectedSndNode != null && selectedSndNode.getUserObject() == selectedDtNode.getUserObject())
						updateSoundTree(null);
					else
						updateSoundTree(selectedSndNode);
					dataTree.removeCurrentNode();
					updateDataTree();
					selectedDtNode = null;
					mainRef.score.display();
					
				}
			}
		});

		importSndMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mainRef.gestionPistes.importSound();
			}
		});

		importDtMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mainRef.gestionPistes.importExternalData();
			}
		});

		updateSndMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				soundTree.clear();
				updateSoundTree(null);
			}
		});

		updateDtMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dataTree.clear();
				updateDataTree();
			}
		});
		clearSndMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int result = JOptionPane.showConfirmDialog(null, "This action will remove all the soundpool waveforms\n"
						+ "(waveform instances in the score will be kept)\n" + "Are you sure ?", "Clear Sounds",
						JOptionPane.OK_CANCEL_OPTION);
				if (result == 0) {
					soundTree.clear();
					mainRef.gestionPistes.soundPool.clear();
					updateSoundTree(null);
				}
				System.gc();
			}
		});
		clearDtMI.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int result = JOptionPane.showConfirmDialog(null, "This action will remove all the soundpool datas\n"
						+ "(data instances in the score will be kept)\n" + "Are you sure ?", "Clear Datas",
						JOptionPane.OK_CANCEL_OPTION);
				if (result == 0) {
					dataTree.clear();
					mainRef.gestionPistes.externalDataPool.clear();
					updateDataTree();
				}
				System.gc();
			}
		});
		info = new JTextPane();
		info.setEditable(false);
		info.setText("Sound/Data informations.");
		info.setFont(font);
		info.setOpaque(false);
		info.setEditable(false);
		info.setDragEnabled(true);
		info.setBorder(BorderFactory.createEmptyBorder(5, 10, 10, 10));
		waveView = new WaveFormRenderer(owner);
		waveView.setBorder(BorderFactory.createLoweredBevelBorder());
		split3 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, soundTree, dataTree);
		split3.setOneTouchExpandable(true);
		split3.setBorder(BorderFactory.createEmptyBorder());
		split2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, split3, info);
		split2.setOneTouchExpandable(true);
		split2.setBorder(BorderFactory.createEmptyBorder());
		split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, false, split2, waveView);
		split.setOneTouchExpandable(false);
		split.setDividerSize(0);
		split.setDividerLocation(250);
		split2.setDividerLocation(220);
		split3.setDividerLocation(115);
		add(split, BorderLayout.CENTER);
		if (supportsDnD()) {
			makeDropTarget(soundTree.getTree());
			makeDropTarget(dataTree.getTree());
		} else
			System.out.println("Drag and drop is not supported with this JVM");
		unfound = new Vector<File>();
	}
/**
 * 
 * @param toSelec Si un noeud particulier doit etre slectionn.
 * 
 */
	public void updateSoundTree(final DefaultMutableTreeNode toSelec) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Vector<HoloWaveForm> sounds = mainRef.gestionPistes.soundPool.getSounds();
				soundTree.clear();
				for (int i=0; i< sounds.size(); i++) {
					HoloWaveForm h = sounds.get(i);
					DefaultMutableTreeNode parent = soundTree.addObject(h);
					HoloWaveForm hwf = (HoloWaveForm) h;
					DefaultMutableTreeNode childNode = null;
					for (Object child : hwf.getLinkedDatas()) {
						if (child != null)
							childNode = soundTree.addObject(parent, child);
					}
					if (toSelec != null) {
						if (toSelec.toString().equals(parent.toString()))
							soundTree.getTree().getSelectionModel().setSelectionPath(new TreePath(childNode.getPath()));
					}else if (i==(sounds.size()-1)){ // expand du dernier import
						soundTree.getTree().expandPath(new TreePath(parent.getPath()));
						soundTree.getTree().getSelectionModel().setSelectionPath(new TreePath(parent.getPath()));
					}
				}
				soundTree.repaint();
			}
		});
	}

	public void updateDataTree() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Vector<HoloExternalData> dataVector = mainRef.gestionPistes.externalDataPool.getSDIFdatas();
				dataTree.clear();
				if (dataVector.size()==0) {
					selectedDtNode = null;
				}else {
					for (int i=0; i<dataVector.size(); i++) {
						HoloExternalData data = dataVector.get(i);
						if (data.getFileType().equalsIgnoreCase("SDIF") || data.getFileType().equalsIgnoreCase("txt")) {
							HoloSDIFdata sdifData = (HoloSDIFdata) data;
							DefaultMutableTreeNode granpa = dataTree.addObject(sdifData);
							for (HoloSDIFdata child : sdifData.children) {
								DefaultMutableTreeNode dad = dataTree.addObject(granpa, child);
								for (HoloSDIFdata child2 : child.children)
									dataTree.addObject(dad,child2);
							}
							if (i==dataVector.size()-1){ // expand du dernier import
								dataTree.getTree().expandPath(new TreePath(granpa.getPath()));
								dataTree.getTree().getSelectionModel().setSelectionPath(new TreePath(granpa.getPath()));
							}
						}else{							
							// TODO other format management
						}
					}
				}
				dataTree.repaint();
			}
		});
	}

	public void clear() {
		soundTree.clear();
		dataTree.clear();
		mainRef.gestionPistes.soundPool.clear();
		mainRef.gestionPistes.externalDataPool.clear();
		unfound.clear();
	}

	public void repaint() {
		super.repaint();
	}

	private void makeDropTarget(final java.awt.Component c) {
		if (c.getParent() != null) {
			// Make drop target && drag source
			@SuppressWarnings("unused")
			final DropTarget dt = new DropTarget(c, this);
			final DragSource ds = new DragSource();
			ds.createDefaultDragGestureRecognizer(c, DnDConstants.ACTION_COPY_OR_MOVE, this);
		}
	}

	public void importSound(File f, boolean verboseError) {
		done = false;
		if (f.getName().length() <= 32) {
			Thread t = new Thread(new SoundFileImporter(f, verboseError));
			t.setPriority(Thread.MAX_PRIORITY);
			t.setName("SF-ImportDeamon");
			t.setDaemon(true);
			t.start();
		} else {
			if (verboseError)
				Ut.alert("Import Error", "Please rename the soundfile, for compatibility reasons with Max/MSP,"
						+ "\nit cannot be longer than 32 characters (including the extension).");
			error = WaveFormReader.PATH_ERROR;
			done = true;
			fine = false;
		}
	}

	public void importData(File f, String importOptions, boolean verboseError) {
		done = false;
		Thread t = new Thread(new DataFileImporter(f, importOptions, verboseError));
		t.setPriority(Thread.MAX_PRIORITY);
		t.setName("SF-ImportDeamon");
		t.setDaemon(true);
		t.start();
	}

	// SELECTION LISTENER
	// *******************************************************
	class WaveformTreeSelectionListener implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent e) {
			selectedSndNode = (DefaultMutableTreeNode) soundTree.getLastSelectedPathComponent();
			sdifDataToDrawFromSndTree.clear();
			if (selectedSndNode == null || selectedSndNode.isRoot()) {
				info.setText("Sound informations.");
				waveView.setHoloWaveForm(null);
			} else {

				Object nodeInfo = selectedSndNode.getUserObject();
				try {
					HoloWaveForm hwf = (HoloWaveForm) nodeInfo;
					info.setText(hwf.getInfo());
					waveView.setHoloWaveForm(hwf);
					// on dessine les tous enfants (data) s'il y en a
					if (!selectedSndNode.isLeaf()) {
						for (Enumeration<?> children = selectedSndNode.children(); children.hasMoreElements();) {
							DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) children.nextElement();
							// si on le dessine pas deja a partir de dataTree
							if (!sdifDataToDraw.contains(childNode.getUserObject()))
								sdifDataToDrawFromSndTree.add((HoloSDIFdata) childNode.getUserObject());
						}
					}
				} catch (ClassCastException cce) {
					try {
						// si on le dessine pas deja a partir de dataTree
						if (!sdifDataToDraw.contains(nodeInfo))
							sdifDataToDrawFromSndTree.add((HoloSDIFdata) nodeInfo);

						// Dessin du pere (une holowaveform)
						try {
							DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) selectedSndNode.getParent();
							HoloWaveForm hwf = (HoloWaveForm) parentNode.getUserObject();
							info.setText(hwf.getInfo());
							waveView.setHoloWaveForm(hwf);
						} catch (ClassCastException cce3) {
							info.setText("Sound informations.");
							waveView.setHoloWaveForm(null);
						}
					} catch (ClassCastException cce2) {
						info.setText("Sound informations.");
						waveView.setHoloWaveForm(null);
					}
				}
			}
			updateWaveView();
		}
	}

	class DataTreeSelectionListener implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent e) {
			TreePath rootPath = new TreePath(dataTree.rootNode);
			boolean b = dataTree.getTree().getSelectionModel().isPathSelected(rootPath);
			if (b == true)
				dataTree.getTree().getSelectionModel().removeSelectionPath(rootPath);

			TreePath[] selectedPaths = dataTree.getSelectionPaths(); // to get every selected node

			sdifDataToDraw.clear();
			if (selectedPaths == null) {
				info.setText("File informations.");
			} else {
				try {
					for (int i = 0; i < selectedPaths.length; i++) {
						selectedDtNode = (DefaultMutableTreeNode) selectedPaths[i].getLastPathComponent();
						HoloSDIFdata hxtdt = (HoloSDIFdata) selectedDtNode.getUserObject();
						if (!sdifDataToDrawFromSndTree.contains(hxtdt))
							sdifDataToDraw.add(hxtdt);
					}

					if (sdifDataToDraw.size() == 1)
						if (sdifDataToDraw.get(0) != null){
							info.setText(sdifDataToDraw.get(0).getInfo());
						}else{
							info.setText("Data informations.");
						}
					else
						info.setText("Multi-Selection..");
				} catch (ClassCastException cce) {
					// other formats
					info.setText("Data informations.");
				}

			}
			updateWaveView();
		}
	}

	// //////////////////////////////////////////////////////////////////////

	public void dragGestureRecognized(DragGestureEvent dge) {
		if (selectedSndNode != null)
			if (dge.getComponent().equals(soundTree.getTree()) && !selectedSndNode.isRoot()) {
				try {
					selectedSndNode = (DefaultMutableTreeNode) soundTree.getLastSelectedPathComponent();
					if (selectedSndNode == null)
						return;
					Object nodeInfo = selectedSndNode.getUserObject();

					HoloWaveForm hwf = (HoloWaveForm) nodeInfo;
					mainRef.score.droppedWaveForm = hwf;
					mainRef.score.acceptDrag = true;

				} catch (ClassCastException cce) {

					mainRef.score.droppedWaveForm = null;

					try {
						selectedSndNode = (DefaultMutableTreeNode) soundTree.getLastSelectedPathComponent();
						Object nodeInfo = selectedSndNode.getUserObject();

						mainRef.score.droppedData = (HoloExternalData) nodeInfo;
						mainRef.score.acceptDrag = true;

					} catch (ClassCastException cce2) {
						mainRef.score.droppedData = null;
						mainRef.score.acceptDrag = false;
					}
				}
			}
		// pour drag du dataTree vers le soundTree ou vers le score
		if (selectedDtNode != null)
			if (dge.getComponent().equals(dataTree.getTree())) {
				try {
					selectedDtNode = (DefaultMutableTreeNode) dataTree.getLastSelectedPathComponent();
					if (selectedDtNode == null) {
						return;
					}
					Object nodeInfo = selectedDtNode.getUserObject();
					HoloExternalData data = (HoloExternalData) nodeInfo;
					if (data.getDataType().equalsIgnoreCase("SDIF")) // drag du plus haut pere interdit
						return;
					this.droppedExternalData = data;
					this.acceptDrag = true;

					mainRef.score.droppedData = data;
					mainRef.score.acceptDrag = true;

				} catch (ClassCastException cce) {
					this.droppedExternalData = null;
					this.acceptDrag = false;
					mainRef.score.droppedData = null;
					mainRef.score.acceptDrag = false;
				}
			}
	}

	private void updateWaveView() {
		Vector<HoloSDIFdata> ar = new Vector<HoloSDIFdata>(sdifDataToDraw);
		ar.addAll(sdifDataToDrawFromSndTree);
		HoloSDIFdata[] hsdifdt = new HoloSDIFdata[ar.size()];
		hsdifdt = ar.toArray(hsdifdt);
		waveView.setHoloSDIFdataTab(hsdifdt);
	}

	public void dragEnter(DropTargetDragEvent dtde) {
		if (isDragOk(dtde))
			dtde.acceptDrag(java.awt.dnd.DnDConstants.ACTION_COPY);
		else if (acceptDrag && droppedExternalData != null) {
			// dtde.getDropTargetContext().dropComplete(true);
		} else
			dtde.rejectDrag();
	}

	public void dragOver(DropTargetDragEvent dtde) {
		if (acceptDrag && droppedExternalData != null)
			try {
				TreePath pathTarget = soundTree.getTree().getPathForLocation(dtde.getLocation().x, dtde.getLocation().y);
				soundTree.getTree().setSelectionPath(pathTarget);

			} catch (NullPointerException e) {

			}
	}

	public void dropActionChanged(DropTargetDragEvent dtde) {
		if (isDragOk(dtde))
			dtde.acceptDrag(java.awt.dnd.DnDConstants.ACTION_COPY);
		else
			dtde.rejectDrag();
	}

	public void dragExit(DropTargetEvent dte) {
	}

	@SuppressWarnings("unchecked")
	public void drop(DropTargetDropEvent dtde) {
		try {
			Transferable tr = dtde.getTransferable();
			if (tr.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
				dtde.acceptDrop(java.awt.dnd.DnDConstants.ACTION_COPY);
				List fileList = (List) tr.getTransferData(DataFlavor.javaFileListFlavor);
				File[] filesTemp = new File[fileList.size()];
				fileList.toArray(filesTemp);
				final File[] files = filesTemp;
				filesDropped(files);
				dtde.getDropTargetContext().dropComplete(true);
			} else if (acceptDrag && droppedExternalData != null) {
				try {
					DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) selectedSndNode.getParent();
					if (parentNode.equals(soundTree.rootNode))
						parentNode = selectedSndNode;

					if (droppedExternalData.getFileType().equalsIgnoreCase("SDIF") || droppedExternalData.getFileType().equalsIgnoreCase("TXT")) {
						HoloSDIFdata hsdifdt = (HoloSDIFdata) droppedExternalData;
						HoloWaveForm wave = (HoloWaveForm) parentNode.getUserObject();
						// on vrifie que l'enfant n'existe pas deja
						if (!wave.containsLinkedData(hsdifdt)) {
							wave.addLinkedData(hsdifdt);
							// on d-selectionne le noeud du dataTree :
							dataTree.getTree().getSelectionModel().clearSelection();
							// update du soundTree avec selection du dernier child
							// ajout  parentNode
							updateSoundTree(parentNode);
						}
					}
				} catch (java.lang.ClassCastException e) {
					this.droppedExternalData = null;
					this.acceptDrag = false;
					dtde.rejectDrop();
				} catch (java.lang.NullPointerException e) {
					this.droppedExternalData = null;
					this.acceptDrag = false;
					dtde.rejectDrop();
				}
				this.droppedExternalData = null;
				this.acceptDrag = false;
			} else
				dtde.rejectDrop();
		} catch (java.io.IOException io) {
			System.out.println("FileDrop: IOException - abort:");
			io.printStackTrace();
			dtde.rejectDrop();
		} catch (java.awt.datatransfer.UnsupportedFlavorException ufe) {
			System.out.println("FileDrop: UnsupportedFlavorException - abort:");
			ufe.printStackTrace();
			dtde.rejectDrop();
		} finally {
		}
	}

	/** Determine if the dragged data is a file list. */
	private boolean isDragOk(final java.awt.dnd.DropTargetDragEvent evt) {
		boolean ok = false;
		DataFlavor[] flavors = evt.getCurrentDataFlavors();
		evt.getSource();

		int i = 0;
		while (!ok && i < flavors.length) {
			// each element is required/guaranteed to be of type java.io.File.
			if (flavors[i].equals(DataFlavor.javaFileListFlavor)) {
				ok = true;
			}
			i++;
		}
		return ok;
	}

	public void filesDropped(File[] files) {
		Vector<File> filesToImport = new Vector<File>(1, 1);
		Vector<File> filesToImport2 = new Vector<File>(1, 1);
		for (File f : files) {
			if (f.isDirectory()) {
				File[] contentFile;
				if (Ut.MAC) {
					MacFileManager OSXFileManager = new MacFileManager(mainRef);
					// allowing sound and data files
					contentFile = OSXFileManager.SoundAndDataFileFilter(f.listFiles());
				} else
					// allowing sound and data files
					contentFile = f.listFiles(mainRef.gestionPistes.sndNdtFilter);

				for (File cf : contentFile)
					if ((cf.getName().substring(cf.getName().lastIndexOf('.') + 1).equalsIgnoreCase("SDIF")) || (cf.getName().substring(cf.getName().lastIndexOf('.') + 1).equalsIgnoreCase("txt"))) {
						filesToImport2.add(cf);
					} else
						filesToImport.add(cf);
			} else {
				if ((f.getName().substring(f.getName().lastIndexOf('.') + 1).equalsIgnoreCase("SDIF")) || (f.getName().substring(f.getName().lastIndexOf('.') + 1).equalsIgnoreCase("txt"))) {
					filesToImport2.add(f);
				} else
					filesToImport.add(f);
			}
		}
		for (File _files : filesToImport) { // changed
			System.out.println("Importing " + _files.getAbsolutePath());
			importSound(_files, true);
		}
		for (File _files : filesToImport2) { // changed
			System.out.println("Importing " + _files.getAbsolutePath());
			importData(_files, null, true);
		}
		mainRef.modify();
	}

	class SoundFileImporter implements Runnable {
		File f;
		boolean verbose = true;
		File cacheFile;

		public SoundFileImporter(File _f, boolean verb) {
			f = _f;
			verbose = verb;
			cacheFile = new File(Ut.dir(Ut.APP_PATH) + WaveFormReader.tmpdir + f.getName() + WaveFormReader.tmpser);
		}

		public void run() {
			if (f != null && ((f.exists() && f.isFile()) || (cacheFile.exists() && cacheFile.isFile()))) {
				done = false;
				fine = false;
				if (!mainRef.gestionPistes.soundPool.contains(new HoloWaveForm(f))) {
					WaveFormReader wfr = new WaveFormReader(f, mainRef);
					try {
						while (!wfr.isDone()) {
							Thread.sleep(20);
							// System.out.println("soundpool-import-waiting");
							// System.out.print('.');
						}
						// System.out.println();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					done = true;
					if (wfr.isFine()) {
						HoloWaveForm[] arr = wfr.getHoloWaveForms();
						for (HoloWaveForm hwf : arr)
							mainRef.gestionPistes.soundPool.add(hwf);
						updateSoundTree(null);
						last = arr[arr.length - 1];
						fine = true;
						if (wfr.getError() == -1)
							Ut.alert("File not found", f.getName() + " can't be found,\nusing it's cached waveform.");
					} else {
						error = wfr.getError();
						errorFileName = wfr.soundFile.getName();
						if (verbose)
							switch (error) {
							case WaveFormReader.NO_ERROR:
								break;
							case WaveFormReader.MONO_ERROR:
								Ut.alert("Error", "Only mono file for instance,\nOnly the left channel has been read.");
								break;
							case WaveFormReader.TYPE_ERROR:
								Ut.alert("Import Error", errorFileName + " is not a supported file type.");
								break;
							case WaveFormReader.PATH_ERROR:
								Ut.alert("Import Error","Please rename the soundfile, for compatibility reasons with Max/MSP," +
										"\nit cannot be longer than 32 characters (including the extension).");
								break;
							case WaveFormReader.PATH2_ERROR:
								Ut.alert("Import Error","Please rename the soundfile, for compatibility reasons with Max/MSP," +
										"\nit cannot" +	" contain blank spaces.");
								break;
							case WaveFormReader.SSDII_NORSRC_ERROR:
								Ut.alert("Import Error", "This Sound Designer II file has no resource and cannot be read.");
								break;
							case WaveFormReader.SSDII_RSRC_ERROR:
								Ut.alert("Import Error", "There are errors in this Sound Designer II resource file.");
								break;
							default:
								Ut.alert("Import Error", "Problem while loading soundfile, aborted.");
								break;
							}
					}
					wfr.stop();
					wfr = null;
					System.gc();
					//System.runFinalization();
				} else {
					done = true;
					fine = true;
					last = new HoloWaveForm(f);
				}
			} else {
				done = true;
				fine = false;
				error = WaveFormReader.FILE_ERROR;
				unfound.add(f);
				if (!choosingfolder) {
					choosingfolder = true;
					Ut.alert("File not found", f.getName() + " can't be found,\neven it's cached waveform.\nLocalize ... ");
				} else {
					try {
						while (choosingfolder) {
							// System.out.println("soundpool-folder-localize-waiting");
							Thread.sleep(500);
						}
					} catch (InterruptedException e) {
					}
				}
				if (soundFolder != null) {
					File[] sounds = soundFolder.listFiles();
					for (int k = unfound.size() - 1; k >= 0; k--) {
						File fil = unfound.get(k);
						for (int i = 0; i < sounds.length; i++) {
							if (sounds[i].getName().equalsIgnoreCase(fil.getName())) {
								mainRef.soundPoolGui.importSound(sounds[i], true);
								unfound.remove(fil);
								mainRef.room.display();
								mainRef.modify();
							}
						}
					}
				} else {
					JFileChooser chooser = new JFileChooser(f.getParentFile().exists() ? f.getParent()
							: mainRef.gestionPistes.holoDirectory);
					chooser.setAcceptAllFileFilterUsed(false);
					chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
					chooser.setDialogTitle("Choose a folder for missing sounds");
					chooser.showDialog(mainRef.soundPoolGui, "Choose this folder for missing sounds");
					if (chooser.getSelectedFile() != null) {
						File tmp = chooser.getSelectedFile();
						if (tmp.exists() && tmp.isDirectory()) {
							soundFolder = tmp;
							choosingfolder = false;
							File[] sounds = soundFolder.listFiles();
							for (int k = unfound.size() - 1; k >= 0; k--) {
								File fil = unfound.get(k);
								for (int i = 0; i < sounds.length; i++) {
									if (sounds[i].getName().equalsIgnoreCase(fil.getName())) {
										mainRef.soundPoolGui.importSound(sounds[i], true);
										unfound.remove(fil);
										mainRef.room.display();
										mainRef.modify();
									}
								}
							}
						}
					}
				}
				if (!unfound.isEmpty())
					Ut.alert("File not found", f.getName() + " can't still be found, aborted.");
			}
		}
	}
	
	public void clearDoneAndFineData(){
		Vector<String> toremove = new Vector<String>();
		for (String data : doneAndFineData){
			if (!mainRef.gestionPistes.externalDataPool.contains(new HoloExternalData(new File(data))))
				toremove.add(data);
		}
		for (String data : toremove)
			doneAndFineData.remove(data);
	}
	
	class DataFileImporter implements Runnable {
		File f;
		String importOptions;
		boolean verbose = true;
		
		public DataFileImporter(File _f, String _importOptions, boolean verb) {
			f = _f;
			importOptions = _importOptions;
			verbose = verb;
		}

		public void run() {
			if (f != null && (f.exists() && f.isFile()) ) {
				done = false;
				fine = false;
				if (!mainRef.gestionPistes.externalDataPool.contains(new HoloExternalData(f))) {
					String extension = f.getName().substring(f.getName().lastIndexOf('.') + 1);
					SoftReference<SDIFreader> sdifReaderRef;
					SDIFreader sdifReader = null;
					SoftReference<TextFileReader> textReaderRef;
					TextFileReader textReader = null;
					if (extension.equalsIgnoreCase("SDIF")) {
						sdifReaderRef = new SoftReference<SDIFreader>(new SDIFreader(f, importOptions, mainRef));
						sdifReader = sdifReaderRef.get();
					} else {
						textReaderRef = new SoftReference<TextFileReader>(new TextFileReader(f));
						textReader = textReaderRef.get();
					}
					if (sdifReader != null) {
						try {
							while (!sdifReader.isDone()) {
								Thread.sleep(20);
							}
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						done = true;
						if (sdifReader.isFine()) {
							HoloSDIFdata[] arr = sdifReader.getHoloSDIFdatas();
							if (arr != null) {
								for (HoloSDIFdata hdt : arr)
									mainRef.gestionPistes.externalDataPool.add(hdt);
								updateDataTree();
								lastXtdt = arr[arr.length - 1];
							}
							doneAndFineData.add(f.getName());
							fine = true;
							if (sdifReader.getError() == -1)
								Ut.alert("File not found", f.getName() + " can't be found,\nusing it's cached waveform.");
						} else {
							error = sdifReader.getError();
							errorFileName = sdifReader.sdifFile.getName();
							if (verbose)
								switch (error) {
								case SDIFreader.NO_ERROR:
									break;
								case SDIFreader.TYPE_ERROR:
									Ut.alert("Import Error", errorFileName + " is not a supported file type.");
									break;
								case TextFileReader.TEXT_FORMAT_ERROR:
									Ut.alert("Import Error", "This text file cannot be imported in holo-Edit.\n" +
											"The data it contains are not properly arranged.\n" +
											"(Columns must be separated by whitespace characters only)");
									break;
								default:
									Ut.alert("Import Error", "Problem while loading datafile, aborted.");
									break;
								}
						}
						sdifReader.stop();
						sdifReader = null;
						System.gc();
					//	System.runFinalization();
					} else if (textReader!=null){
						try {
							while (!textReader.isDone()) {
								Thread.sleep(20);
							}
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						done = true;
						if (textReader.isFine()) {
							HoloSDIFdata[] arr = textReader.getHoloSDIFdatas();
							if (arr != null) {
								for (HoloSDIFdata hdt : arr)
									mainRef.gestionPistes.externalDataPool.add(hdt);
								updateDataTree();
								lastXtdt = arr[arr.length - 1];
							}
							fine = true;
							doneAndFineData.add(f.getName());
							if (textReader.getError() == -1)
								Ut.alert("File not found", f.getName() + " can't be found,\nusing it's cached waveform.");
						} else {
							error = textReader.getError();
							errorFileName = textReader.textFile.getName();
							if (verbose)
								switch (error) {
								case SDIFreader.NO_ERROR:
									break;
								case SDIFreader.TYPE_ERROR:
									Ut.alert("Import Error", errorFileName + " is not a supported file type.");
									break;
								case TextFileReader.TEXT_FORMAT_ERROR:
									Ut.alert("Import Error", "This text file cannot be imported in holo-Edit.\n" +
											"Be sure the data it contains are properly arranged.\n" +
											"(Columns must be separated by whitespace characters only)");
									break;
								default:
									Ut.alert("Import Error", "Problem while loading datafile, aborted.");
									break;
								}
						}
						textReader.stop();
						textReader = null;
						System.gc();
						//System.runFinalization();
					}
				} else {
					doneAndFineData.add(f.getName());
					done = true;
					fine=true;
					lastXtdt = new HoloExternalData(f);
				}
			} else {
				done = true;
				fine = false;
				error = SDIFreader.FILE_ERROR;
				unfound.add(f);
				if (!choosingfolder) {
					choosingfolder = true;
					Ut.alert("File not found", f.getName() + " can't be found.\nLocalize ... ");
				} else {
					try {
						while (choosingfolder) {
							// System.out.println("soundpool-folder-localize-waiting");
							Thread.sleep(500);
						}
					} catch (InterruptedException e) {
					}
				}
				if (externalDataFolder != null) {
					File[] datas = externalDataFolder.listFiles();
					for (int k = unfound.size() - 1; k >= 0; k--) {
						File fil = unfound.get(k);
						for (int i = 0; i < datas.length; i++) {
							if (datas[i].getName().equalsIgnoreCase(fil.getName())) {
								mainRef.soundPoolGui.importData(datas[i], importOptions, true);
								unfound.remove(fil);
								mainRef.room.display();
								mainRef.modify();
							}
						}
					}
				} else {
					JFileChooser chooser = new JFileChooser(f.getParentFile().exists() ? f.getParent()
							: mainRef.gestionPistes.holoDirectory);
					chooser.setAcceptAllFileFilterUsed(false);
					chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
					chooser.setDialogTitle("Choose a folder for missing datas");
					chooser.showDialog(mainRef.soundPoolGui, "Choose this folder for missing datas");
					if (chooser.getSelectedFile() != null) {
						File tmp = chooser.getSelectedFile();
						if (tmp.exists() && tmp.isDirectory()) {
							dataFolder = tmp;
							choosingfolder = false;
							File[] datas = dataFolder.listFiles();
							for (int k = unfound.size() - 1; k >= 0; k--) {
								File fil = unfound.get(k);
								for (int i = 0; i < datas.length; i++) {
									if (datas[i].getName().equalsIgnoreCase(fil.getName())) {
										mainRef.soundPoolGui.importData(datas[i], importOptions, true);
										unfound.remove(fil);
										mainRef.room.display();
										mainRef.modify();
									}
								}
							}
						}
					}
				}
				if (!unfound.isEmpty())
					Ut.alert("File not found", f.getName() + " can't still be found, aborted.");
			}
		}
	}

	public int getSplitDivPos() {
		return split2.getDividerLocation();
	}

	public void setSplitDivPos(int i) {
		split2.setDividerLocation(i);
	}

	public static boolean supportsDnD() { // Static Boolean
		if (supportsDnD == null) {
			boolean support = false;
			try {
				Class.forName("java.awt.dnd.DnDConstants");
				support = true;
			} // end try
			catch (Exception e) {
				support = false;
			} // end catch
			supportsDnD = new Boolean(support);
		} // end if: first time through
		return supportsDnD.booleanValue();
	} // end supportsDnD

	public String toString() {
		return "\t<soundpool  split=\"" + getSplitDivPos() + "\"" + super.toString();
	}

	public void componentResized(ComponentEvent e) {
		super.componentResized(e);
		split.setDividerLocation(250);
	}
}